"""Image Hash Command-line tool."""

from pathlib import Path
from typing import List
import cv2
import numpy as np


class ImagePHash:  # pylint: disable=too-many-instance-attributes
    """ImagePHash Class."""

    def __init__(self, filename: str, mode: str, flip_v: bool, flip_h: bool):
        """__init__."""
        self.filename = filename
        self.mode = mode
        self.hash: List[np.ndarray] = []
        self.flip_h = flip_h
        self.flip_v = flip_v

        # parameters for some hash functions
        self.block_mean_hash_mode = 0
        self.marr_hildreth_hash_alpha = 2.0
        self.marr_hildreth_hash_scale = 1.0
        self.radial_variance_hash_sigma = 1.0
        self.radial_variance_hash_num_of_angle_line = 180

    def __str__(self) -> str:
        """__str__."""
        return f"{self.filename}({self.mode})"

    def compute_hash(self, img: np.array) -> np.array:  # type: ignore
        """compute_hash."""
        if self.mode == "pHash":
            return cv2.img_hash.pHash(img)
        if self.mode == "averageHash":
            return cv2.img_hash.averageHash(img)
        if self.mode == "blockMeanHash":
            # return cv2.img_hash.blockMeanHash(img)
            return cv2.img_hash.blockMeanHash(
                img, mode=self.block_mean_hash_mode
            )
        if self.mode == "marrHildrethHash":
            # return cv2.img_hash.marrHildrethHash(img)
            return cv2.img_hash.marrHildrethHash(
                img,
                alpha=self.marr_hildreth_hash_alpha,
                scale=self.marr_hildreth_hash_scale,
            )
        if self.mode == "radialVarianceHash":
            # return cv2.img_hash.radialVarianceHash(img)
            return cv2.img_hash.radialVarianceHash(
                img,
                sigma=self.radial_variance_hash_sigma,
                numOfAngleLine=self.radial_variance_hash_num_of_angle_line,
            )
        if self.mode == "colorMomentHash":
            return cv2.img_hash.colorMomentHash(img)
        raise ValueError("Hash mode not found", self.mode)

    def image_hash_file(self) -> np.ndarray:
        """img-phash-tools from filename."""
        my_file = Path(self.filename)
        if not my_file.is_file():
            raise ValueError("Filename not found", self.filename)
        img = cv2.imread(self.filename)
        h = self.compute_hash(img)
        self.hash.append(
            h[0]  # type: ignore # pylint: disable=unsubscriptable-object
        )
        if self.flip_v:
            img = cv2.flip(img, 0)
            h = self.compute_hash(img)
            self.hash.append(
                h[0]  # type: ignore # pylint: disable=unsubscriptable-object
            )
        if self.flip_h:
            img = cv2.flip(img, 1)
            h = self.compute_hash(img)
            self.hash.append(
                h[0]  # type: ignore # pylint: disable=unsubscriptable-object
            )
        if self.flip_v and self.flip_h:
            img = cv2.flip(img, 0)
            h = self.compute_hash(img)
            self.hash.append(
                h[0]  # type: ignore # pylint: disable=unsubscriptable-object
            )
        return self.hash[0]

    @staticmethod
    def hash_to_str(f_hash: np.ndarray) -> str:
        """hash_to_str."""
        return str(
            int.from_bytes(f_hash.tobytes(), byteorder="big", signed=False)
        )

    @staticmethod
    def str_to_hash(f_hash: str, mode: str) -> np.ndarray:
        """str_to_hash."""
        cnt = 8
        if mode == "pHash":
            cnt = 8
        elif mode == "averageHash":
            cnt = 8
        elif mode == "blockMeanHash":
            cnt = 32
        elif mode == "marrHildrethHash":
            cnt = 72
        elif mode == "radialVarianceHash":
            cnt = 40
        elif mode == "colorMomentHash":
            # return np.frombuffer(int(f_hash).to_bytes(42,
            # "big", signed=False), dtype=np.float64)
            # return np.frombuffer(float(f_hash).to_bytes(
            # 42, "big", signed=False), dtype=np.float64)
            # Not yet implemented
            raise ValueError("Hash mode not yet implemented", mode)
        else:
            raise ValueError("Hash mode not known", mode)
        return np.frombuffer(
            int(f_hash).to_bytes(cnt, "big", signed=False), dtype=np.uint8
        )

    @staticmethod
    def min_hash_distance(
        a: List[np.ndarray], b: List[np.ndarray], verbose: bool = False
    ) -> float:
        """min_hash_distance."""
        min_dist = -1.0
        for _, h1 in enumerate(a):
            for _, h2 in enumerate(b):
                dist = ImagePHash.hamming_distance(h1, h2)
                if min_dist == -1.0:
                    min_dist = dist
                min_dist = min(dist, min_dist)
                if verbose:
                    print(str(dist) + ";", end="")
        if verbose:
            print("")
            print(str(min_dist) + ";")
        return min_dist

    @staticmethod
    def hamming_distance_v3(a: np.ndarray, b: np.ndarray) -> int:
        """hamming_distance_v3."""
        r = (1 << np.arange(8))[:, None]
        return int(np.count_nonzero((a & r) != (b & r)))

    @staticmethod
    def hamming_distance_v2(a: np.ndarray, b: np.ndarray) -> int:
        """hamming_distance_v2."""
        r = (1 << np.arange(8))[:, None]
        return int(np.count_nonzero((np.bitwise_xor(a, b) & r) != 0))

    @staticmethod
    def hamming_distance(a: np.ndarray, b: np.ndarray) -> float:
        """hamming_distance."""
        return float(str(cv2.norm(a, b, normType=cv2.NORM_HAMMING)))
        # return float(str(cv2.norm(a, b, normType=cv2.NORM_HAMMING2)))



"""Image Hash Command-line tool."""

import typer
from typing_extensions import Annotated
from imgphash.image_phash import ImagePHash


def imgphash_cli(  # pylint: disable=too-many-arguments,too-many-locals
    filename: Annotated[
        str,
        typer.Argument(help="The file name."),
    ] = ".",
    *,
    mode: Annotated[
        str,
        typer.Option(
            help="The hash mode : [averageHash, blockMeanHash, "
            "colorMomentHash, marrHildrethHash, pHash, radialVarianceHash]."
        ),
    ] = "pHash",
    flip_v: Annotated[
        bool,
        typer.Option(
            help="Flip the image vertically (so hash will be flip resistant)."
        ),
    ] = False,
    flip_h: Annotated[
        bool,
        typer.Option(
            help="Flip the image horizontally (so hash will be flip "
            "resistant)."
        ),
    ] = False,
    verbose: Annotated[
        bool,
        typer.Option(help="Print more informations"),
    ] = False,
    compare: Annotated[
        str,
        typer.Option(
            help="Compare to an other image filename and return distance."
        ),
    ] = "",
    block_mean_hash_mode: Annotated[
        int,
        typer.Option(help="block_mean_hash_mode int, default:0"),
    ] = 0,
    marr_hildreth_hash_alpha: Annotated[
        float,
        typer.Option(help="marr_hildreth_hash_alpha float, default:2.0"),
    ] = 2.0,
    marr_hildreth_hash_scale: Annotated[
        float,
        typer.Option(help="marr_hildreth_hash_scale float, default:1.0"),
    ] = 1.0,
    radial_variance_hash_sigma: Annotated[
        float,
        typer.Option(help="radial_variance_hash_sigma float, default:1.0"),
    ] = 1.0,
    radial_variance_hash_num_of_angle_line: Annotated[
        int,
        typer.Option(
            help="radial_variance_hash_num_of_angle_line int, default:180"
        ),
    ] = 180,
) -> None:
    """
    Use to print the perceptual hash of an image.

    See https://www.phash.org or

    https://www.phash.org/docs/pubs/thesis_zauner.pdf

    for more informations on the algorithms.

    It return the hashes into an integer, you can use hamming
    distance on his bits to find similar images.

    With --compare option, it will return the hamming distance
    between the two images.
    """
    img_hash = ImagePHash(
        filename=filename, mode=mode, flip_v=flip_v, flip_h=flip_h
    )
    img_hash.block_mean_hash_mode = block_mean_hash_mode
    img_hash.marr_hildreth_hash_scale = marr_hildreth_hash_scale
    img_hash.marr_hildreth_hash_alpha = marr_hildreth_hash_alpha
    img_hash.radial_variance_hash_num_of_angle_line = (
        radial_variance_hash_num_of_angle_line
    )
    img_hash.radial_variance_hash_sigma = radial_variance_hash_sigma
    f_hash = img_hash.image_hash_file()
    if not compare or verbose:
        print(ImagePHash.hash_to_str(f_hash) + ";", end="")
        for i in range(1, len(img_hash.hash)):
            print(ImagePHash.hash_to_str(img_hash.hash[i]) + ";", end="")
        print(mode + ";")
    if compare != "":
        img_hash2 = ImagePHash(
            filename=compare, mode=mode, flip_v=flip_v, flip_h=flip_h
        )
        img_hash2.block_mean_hash_mode = block_mean_hash_mode
        img_hash2.marr_hildreth_hash_scale = marr_hildreth_hash_scale
        img_hash2.marr_hildreth_hash_alpha = marr_hildreth_hash_alpha
        img_hash2.radial_variance_hash_num_of_angle_line = (
            radial_variance_hash_num_of_angle_line
        )
        img_hash2.radial_variance_hash_sigma = radial_variance_hash_sigma
        f_hash2 = img_hash2.image_hash_file()
        if verbose:
            print(ImagePHash.hash_to_str(f_hash2) + ";", end="")
            for j in range(1, len(img_hash2.hash)):
                print(ImagePHash.hash_to_str(img_hash2.hash[j]) + ";", end="")
            print(mode + ";")

        dist = ImagePHash.min_hash_distance(
            img_hash.hash, img_hash2.hash, verbose=verbose
        )
        if not verbose:
            print(str(dist) + ";")


app = typer.Typer()
app.command()(imgphash_cli)


if __name__ == "__main__":
    app()



"""Image Hash Command-line tool."""

from pathlib import Path

import random
import typer
from typing_extensions import Annotated
from imgphash.image_phash import ImagePHash

from dupimg.cli import run_fast_scandir


def qsimg_cli(
    # pylint: disable=too-many-arguments,too-many-locals
    directory: Annotated[
        str,
        typer.Argument(help="The directory name."),
    ] = ".",
    *,
    reference: Annotated[
        str,
        typer.Argument(
            help="Reference file name. If given, it will "
            "sort only by similarity with this image."
        ),
    ] = "",
    rand: Annotated[
        bool,
        typer.Option(help="Use a random reference"),
    ] = True,
    seed: Annotated[
        int,
        typer.Option(help="Seed for the random generator"),
    ] = -1,
    mode: Annotated[
        str,
        typer.Option(
            help="The hash mode : [averageHash, blockMeanHash, "
            "colorMomentHash, marrHildrethHash, pHash, radialVarianceHash]."
        ),
    ] = "pHash",
    recurse: Annotated[
        bool,
        typer.Option(help="Recurse over folders."),
    ] = True,
    flip_v: Annotated[
        bool,
        typer.Option(
            help="Flip the image vertically (so hash will be flip resistant)."
        ),
    ] = False,
    flip_h: Annotated[
        bool,
        typer.Option(
            help="Flip the image horizontally (so hash will be flip "
            "resistant)."
        ),
    ] = False,
    block_mean_hash_mode: Annotated[
        int,
        typer.Option(help="block_mean_hash_mode int, default:0"),
    ] = 0,
    marr_hildreth_hash_alpha: Annotated[
        float,
        typer.Option(help="marr_hildreth_hash_alpha float, default:2.0"),
    ] = 2.0,
    marr_hildreth_hash_scale: Annotated[
        float,
        typer.Option(help="marr_hildreth_hash_scale float, default:1.0"),
    ] = 1.0,
    radial_variance_hash_sigma: Annotated[
        float,
        typer.Option(help="radial_variance_hash_sigma float, default:1.0"),
    ] = 1.0,
    radial_variance_hash_num_of_angle_line: Annotated[
        int,
        typer.Option(
            help="radial_variance_hash_num_of_angle_line int, default:180"
        ),
    ] = 180,
    verbose: Annotated[
        bool,
        typer.Option(help="Print more values."),
    ] = False,
) -> None:
    """
    Quick Find Similar images with perceptual hash algorithms.

    It returns an ordered list by similarity with the reference image.

    In random mode, it will find a random reference.

    """
    if reference != "" and not Path(reference).is_file():
        raise ValueError("Filename not found", reference)
    if reference != "" and Path(reference).is_file():
        rand = False
    if not Path(directory).is_dir():
        raise ValueError("Directory not found", directory)
    _, files = run_fast_scandir(
        directory, [".jpg", ".jpeg", ".png", ".gif", ".webp", ".bmp"], recurse
    )
    if seed != -1:
        random.seed(seed)
    if rand:
        reference = files[random.randint(0, len(files))]  # nosec B311
    my_file = Path(reference)
    if not my_file.is_file():
        raise ValueError("File not found", reference)
    img_hash = ImagePHash(
        filename=reference, mode=mode, flip_v=flip_v, flip_h=flip_h
    )
    img_hash.block_mean_hash_mode = block_mean_hash_mode
    img_hash.marr_hildreth_hash_scale = marr_hildreth_hash_scale
    img_hash.marr_hildreth_hash_alpha = marr_hildreth_hash_alpha
    img_hash.radial_variance_hash_num_of_angle_line = (
        radial_variance_hash_num_of_angle_line
    )
    img_hash.radial_variance_hash_sigma = radial_variance_hash_sigma
    img_hash.image_hash_file()
    ref_hashes = img_hash.hash

    files_dist: dict[str, float] = {}
    for _, file in enumerate(files):
        img_hash = ImagePHash(
            filename=file, mode=mode, flip_v=flip_v, flip_h=flip_h
        )
        img_hash.block_mean_hash_mode = block_mean_hash_mode
        img_hash.marr_hildreth_hash_scale = marr_hildreth_hash_scale
        img_hash.marr_hildreth_hash_alpha = marr_hildreth_hash_alpha
        img_hash.radial_variance_hash_num_of_angle_line = (
            radial_variance_hash_num_of_angle_line
        )
        img_hash.radial_variance_hash_sigma = radial_variance_hash_sigma
        img_hash.image_hash_file()
        files_dist[file] = ImagePHash.min_hash_distance(
            ref_hashes, img_hash.hash, verbose=False
        )
    files_dist = dict(sorted(files_dist.items(), key=lambda item: item[1]))
    for k, v in files_dist.items():
        if verbose:
            print(f"{k};{v:.0f};")
        else:
            print(k)


app = typer.Typer()
app.command()(qsimg_cli)


if __name__ == "__main__":
    app()



"""Image Hash Command-line tool."""

from pathlib import Path

import numpy as np
import typer
from typing_extensions import Annotated
from imgphash.image_phash import ImagePHash

from dupimg.cli import run_fast_scandir


def matrix_to_csv(mat: dict[str, dict[str, float]]) -> None:
    """matrix_to_csv."""
    # print header
    print(";", end="")
    for sh1, _ in mat.items():
        print(sh1 + ";", end="")
    print("")
    for sh1, _ in mat.items():
        print(sh1 + ";", end="")
        for sh2, _ in mat.items():
            # diagonalisation of matrix
            if sh1 in mat and sh2 in mat[sh1]:
                print(f"{mat[sh1][sh2]:.0f};", end="")
            else:
                print(f"{mat[sh2][sh1]:.0f};", end="")
        print("")


def simg_cli(
    # pylint: disable=too-many-arguments,too-many-locals
    directory: Annotated[
        str,
        typer.Argument(help="The directory name."),
    ] = ".",
    *,
    mode: Annotated[
        str,
        typer.Option(
            help="The hash mode : [averageHash, blockMeanHash, "
            "colorMomentHash, marrHildrethHash, pHash, radialVarianceHash]."
        ),
    ] = "pHash",
    recurse: Annotated[
        bool,
        typer.Option(help="Recurse over folders."),
    ] = True,
    flip_v: Annotated[
        bool,
        typer.Option(
            help="Flip the image vertically (so hash will be flip resistant)."
        ),
    ] = False,
    flip_h: Annotated[
        bool,
        typer.Option(
            help="Flip the image horizontally (so hash will be flip "
            "resistant)."
        ),
    ] = False,
    block_mean_hash_mode: Annotated[
        int,
        typer.Option(help="block_mean_hash_mode int, default:0"),
    ] = 0,
    marr_hildreth_hash_alpha: Annotated[
        float,
        typer.Option(help="marr_hildreth_hash_alpha float, default:2.0"),
    ] = 2.0,
    marr_hildreth_hash_scale: Annotated[
        float,
        typer.Option(help="marr_hildreth_hash_scale float, default:1.0"),
    ] = 1.0,
    radial_variance_hash_sigma: Annotated[
        float,
        typer.Option(help="radial_variance_hash_sigma float, default:1.0"),
    ] = 1.0,
    radial_variance_hash_num_of_angle_line: Annotated[
        int,
        typer.Option(
            help="radial_variance_hash_num_of_angle_line int, default:180"
        ),
    ] = 180,
) -> None:
    """
    Find Similar images with perceptual hash algorithms.

    It returns a distance matrix between found images.

    """
    my_file = Path(directory)
    if not my_file.is_dir():
        raise ValueError("Directory not found", directory)
    _, files = run_fast_scandir(
        directory, [".jpg", ".jpeg", ".png", ".gif", ".webp", ".bmp"], recurse
    )
    files_hashes: dict[str, list[np.ndarray]] = {}

    for _, file in enumerate(files):
        img_hash = ImagePHash(
            filename=file, mode=mode, flip_v=flip_v, flip_h=flip_h
        )
        img_hash.block_mean_hash_mode = block_mean_hash_mode
        img_hash.marr_hildreth_hash_scale = marr_hildreth_hash_scale
        img_hash.marr_hildreth_hash_alpha = marr_hildreth_hash_alpha
        img_hash.radial_variance_hash_num_of_angle_line = (
            radial_variance_hash_num_of_angle_line
        )
        img_hash.radial_variance_hash_sigma = radial_variance_hash_sigma
        img_hash.image_hash_file()
        files_hashes[file] = img_hash.hash

    hashes_distance: dict[str, dict[str, float]] = {}
    for sh1, hashes in files_hashes.items():
        for sh2, hashes2 in files_hashes.items():
            # diagonalisation of matrix
            if sh2 in hashes_distance:
                continue
            if sh1 not in hashes_distance:
                hashes_distance[sh1] = {}
            if sh1 == sh2:
                hashes_distance[sh1][sh2] = 0.0
            else:
                hashes_distance[sh1][sh2] = ImagePHash.min_hash_distance(
                    hashes, hashes2, verbose=False
                )
    matrix_to_csv(hashes_distance)


app = typer.Typer()
app.command()(simg_cli)


if __name__ == "__main__":
    app()



"""Image Hash Command-line tool."""

import os
from pathlib import Path
from typing import Tuple

import typer
from typing_extensions import Annotated
from imgphash.image_phash import ImagePHash


def run_fast_scandir(
    directory: str, ext: list[str], recurse: bool = True
) -> Tuple[list[str], list[str]]:
    """run_fast_scandir."""
    subfolders: list[str] = []
    files: list[str] = []
    for f in os.scandir(directory):
        if recurse:
            if f.is_dir():
                subfolders.append(f.path)
        if f.is_file():
            if os.path.splitext(f.name)[1].lower() in ext:
                files.append(f.path)
    if recurse:
        for directory2 in list(subfolders):
            sf, fi = run_fast_scandir(directory2, ext)
            subfolders.extend(sf)
            files.extend(fi)
    return subfolders, files


def dupimg_cli(  # pylint: disable=too-many-arguments,too-many-locals
    directory: Annotated[
        str,
        typer.Argument(help="The directory name."),
    ] = ".",
    *,
    mode: Annotated[
        str,
        typer.Option(
            help="The hash mode : [averageHash, blockMeanHash, "
            "colorMomentHash, marrHildrethHash, pHash, radialVarianceHash]."
        ),
    ] = "pHash",
    recurse: Annotated[
        bool,
        typer.Option(help="Recurse over folders."),
    ] = True,
    flip_v: Annotated[
        bool,
        typer.Option(
            help="Flip the image vertically (so hash will be flip resistant)."
        ),
    ] = False,
    flip_h: Annotated[
        bool,
        typer.Option(
            help="Flip the image horizontally (so hash will be flip "
            "resistant)."
        ),
    ] = False,
    block_mean_hash_mode: Annotated[
        int,
        typer.Option(help="block_mean_hash_mode int, default:0"),
    ] = 0,
    marr_hildreth_hash_alpha: Annotated[
        float,
        typer.Option(help="marr_hildreth_hash_alpha float, default:2.0"),
    ] = 2.0,
    marr_hildreth_hash_scale: Annotated[
        float,
        typer.Option(help="marr_hildreth_hash_scale float, default:1.0"),
    ] = 1.0,
    radial_variance_hash_sigma: Annotated[
        float,
        typer.Option(help="radial_variance_hash_sigma float, default:1.0"),
    ] = 1.0,
    radial_variance_hash_num_of_angle_line: Annotated[
        int,
        typer.Option(
            help="radial_variance_hash_num_of_angle_line int, default:180"
        ),
    ] = 180,
) -> None:
    """
    Find duplicates images with perceptual hash algorithms.

    It returns a list of filenames separated by a ';'

    On the same line, the files are similar.

    On other lines, there is other duplicate files.

    """
    my_file = Path(directory)
    if not my_file.is_dir():
        raise ValueError("Directory not found", directory)
    _, files = run_fast_scandir(
        directory, [".jpg", ".jpeg", ".png", ".gif", ".webp", ".bmp"], recurse
    )
    files_hash: dict[str, dict[str, bool]] = {}
    duplicates: dict[str, dict[str, bool]] = {}
    for _, file in enumerate(files):
        img_hash = ImagePHash(
            filename=file, mode=mode, flip_v=flip_v, flip_h=flip_h
        )
        img_hash.block_mean_hash_mode = block_mean_hash_mode
        img_hash.marr_hildreth_hash_scale = marr_hildreth_hash_scale
        img_hash.marr_hildreth_hash_alpha = marr_hildreth_hash_alpha
        img_hash.radial_variance_hash_num_of_angle_line = (
            radial_variance_hash_num_of_angle_line
        )
        img_hash.radial_variance_hash_sigma = radial_variance_hash_sigma
        img_hash.image_hash_file()
        for _, f_hash in enumerate(img_hash.hash):
            str_hash = ImagePHash.hash_to_str(f_hash)
            if str_hash in files_hash:
                if file not in files_hash[str_hash]:
                    files_hash[str_hash].update({file: True})
                    duplicates[str_hash] = files_hash[str_hash]
            else:
                files_hash[str_hash] = {file: True}
    # This list is used when there where flipped images,
    # two hashes will give the same list, and we don't print it
    printed: dict[str, bool] = {}
    for _, k in duplicates.items():
        prt = ""
        for file, _ in k.items():
            prt = prt + file + ";"
        if prt not in printed:
            print(prt)
            printed.update({prt: True})


app = typer.Typer()
app.command()(dupimg_cli)


if __name__ == "__main__":
    app()



#!/usr/bin/python

"""Image Hash Command-line tool."""
import os
from pathlib import Path
import cv2
import numpy as np
import pytest
from imgphash.image_phash import ImagePHash


@pytest.fixture
def make_images():
    if not os.path.exists("./tests/datas"):
        os.makedirs("./tests/datas")
    if not os.path.exists("./tests/datas/2"):
        os.makedirs("./tests/datas/2")
    newImage = np.zeros((768, 768, 3), np.uint8)
    cv2.imwrite(
        "./tests/datas/imageBlack.png",
        newImage,
        [cv2.IMWRITE_PNG_COMPRESSION, 0],
    )
    if not Path("./tests/datas/imageBlack.png").is_file():
        cv2.imwrite(
            "./tests/datas/imageBlack.png",
            newImage,
            [cv2.IMWRITE_PNG_COMPRESSION, 0],
        )
    if not Path("./tests/datas/imageRedBlue.png").is_file():
        # blue channel
        newImage[0 : 768 // 2, :, 0] = 255
        newImage[768 // 2 : 768, :, 0] = 0
        # green channel
        newImage[0 : 768 // 2, :, 1] = 0
        newImage[768 // 2 : 768, :, 1] = 0
        # red images
        newImage[0 : 768 // 2, :, 2] = 0
        newImage[768 // 2 : 768, :, 2] = 255
        cv2.imwrite(
            "./tests/datas/imageRedBlue.png",
            newImage,
            [cv2.IMWRITE_PNG_COMPRESSION, 0],
        )
    if not Path("./tests/datas/imageWhite.png").is_file():
        # blue channel
        newImage[:, :, 0] = 255
        # green channel
        newImage[:, :, 1] = 255
        # red channel
        newImage[:, :, 2] = 255
        cv2.imwrite(
            "./tests/datas/imageWhite.png",
            newImage,
            [cv2.IMWRITE_PNG_COMPRESSION, 0],
        )

    center_point: tuple[int, int] = (200, 200)
    radius_circle: int = 100
    color_line: tuple[int, int, int] = (0, 0, 255)
    line_thickness: int = -1
    if not Path("./tests/datas/imageWhiteCircle1.png").is_file():
        cv2.circle(
            newImage, center_point, radius_circle, color_line, line_thickness
        )
        cv2.imwrite(
            "./tests/datas/imageWhiteCircle1.png",
            newImage,
            [cv2.IMWRITE_PNG_COMPRESSION, 0],
        )
        newImage = cv2.flip(newImage, 0)
        cv2.imwrite(
            "./tests/datas/imageWhiteCircle1fv.png",
            newImage,
            [cv2.IMWRITE_PNG_COMPRESSION, 0],
        )
        newImage = cv2.flip(newImage, 1)
        cv2.imwrite(
            "./tests/datas/2/imageWhiteCircle1fvfh.png",
            newImage,
            [cv2.IMWRITE_PNG_COMPRESSION, 0],
        )
        newImage = cv2.flip(newImage, 0)
        cv2.imwrite(
            "./tests/datas/imageWhiteCircle1fh.png",
            newImage,
            [cv2.IMWRITE_PNG_COMPRESSION, 0],
        )
    if not Path("./tests/datas/imageWhiteCircle2.png").is_file():
        newImage[:, :, 0] = 255
        newImage[:, :, 1] = 255
        newImage[:, :, 2] = 255
        cv2.circle(newImage, center_point, 150, color_line, line_thickness)
        cv2.imwrite(
            "./tests/datas/imageWhiteCircle2.png",
            newImage,
            [cv2.IMWRITE_PNG_COMPRESSION, 0],
        )
    if not Path("./tests/datas/imageWhiteCircle3.png").is_file():
        newImage[:, :, 0] = 255
        newImage[:, :, 1] = 255
        newImage[:, :, 2] = 255
        cv2.circle(newImage, center_point, 150, color_line, 10)
        cv2.imwrite(
            "./tests/datas/imageWhiteCircle3.png",
            newImage,
            [cv2.IMWRITE_PNG_COMPRESSION, 0],
        )
    if not Path("./tests/datas/imageBlackCircle1.png").is_file():
        newImage[:, :, 0] = 0
        newImage[:, :, 1] = 0
        newImage[:, :, 2] = 0
        cv2.circle(
            newImage, (400, 300), radius_circle, color_line, line_thickness
        )
        cv2.imwrite(
            "./tests/datas/imageBlackCircle1.png",
            newImage,
            [cv2.IMWRITE_PNG_COMPRESSION, 0],
        )
    if not Path("./tests/datas/imageBlackRectangle1.png").is_file():
        newImage[:, :, 0] = 0
        newImage[:, :, 1] = 0
        newImage[:, :, 2] = 0
        cv2.rectangle(
            newImage, (300, 200), (500, 400), color=(255, 0, 0), thickness=10
        )
        cv2.imwrite(
            "./tests/datas/imageBlackRectangle1.png",
            newImage,
            [cv2.IMWRITE_PNG_COMPRESSION, 0],
        )
    if not Path("./tests/datas/imageBlackRectangle2.png").is_file():
        newImage[:, :, 0] = 0
        newImage[:, :, 1] = 0
        newImage[:, :, 2] = 0
        cv2.rectangle(
            newImage, (300, 200), (500, 400), color=(255, 0, 0), thickness=-1
        )
        cv2.imwrite(
            "./tests/datas/imageBlackRectangle2.png",
            newImage,
            [cv2.IMWRITE_PNG_COMPRESSION, 0],
        )
    return True


def test_toStr(make_images):
    assert make_images == True
    img_hash = ImagePHash(
        filename="./tests/datas/imageBlackRectangle2.png",
        mode="pHash",
        flip_v=False,
        flip_h=False,
    )
    assert str(img_hash) == "./tests/datas/imageBlackRectangle2.png(pHash)"


def test_bad_mode(make_images):
    assert make_images == True
    img_hash = ImagePHash(
        filename="./tests/datas/imageBlackRectangle2.png",
        mode="sdfdsfsdf",
        flip_v=False,
        flip_h=False,
    )
    with pytest.raises(Exception):
        f_hash = img_hash.image_hash_file()
        assert ImagePHash.hash_to_str(f_hash) == "11067990150089382910"


def test_bad_filename(make_images):
    assert make_images == True
    img_hash = ImagePHash(
        filename="/tmp/dataNotFound.png",
        mode="pHash",
        flip_v=False,
        flip_h=False,
    )
    with pytest.raises(Exception):
        f_hash = img_hash.image_hash_file()
        assert ImagePHash.hash_to_str(f_hash) == "11067990150089382910"


def test_pHash(make_images):
    assert make_images == True
    img_hash = ImagePHash(
        filename="./tests/datas/imageBlackRectangle2.png",
        mode="pHash",
        flip_v=False,
        flip_h=False,
    )
    f_hash = img_hash.image_hash_file()
    assert ImagePHash.hash_to_str(f_hash) == "11067990150089382910"
    assert (
        ImagePHash.hash_to_str(
            ImagePHash.str_to_hash(ImagePHash.hash_to_str(f_hash), "pHash")
        )
        == "11067990150089382910"
    )
    with pytest.raises(Exception):
        assert (
            ImagePHash.hash_to_str(
                ImagePHash.str_to_hash(ImagePHash.hash_to_str(f_hash), "TEST")
            )
            == "11067990150089382910"
        )

    img_hash = ImagePHash(
        filename="./tests/datas/imageBlackRectangle1.png",
        mode="pHash",
        flip_v=False,
        flip_h=False,
    )
    f_hash = img_hash.image_hash_file()
    assert ImagePHash.hash_to_str(f_hash) == "8319147471542838937"
    img_hash = ImagePHash(
        filename="./tests/datas/imageBlackCircle1.png",
        mode="pHash",
        flip_v=False,
        flip_h=False,
    )
    f_hash = img_hash.image_hash_file()
    assert ImagePHash.hash_to_str(f_hash) == "11067990150089382374"
    img_hash = ImagePHash(
        filename="./tests/datas/imageBlack.png",
        mode="pHash",
        flip_v=False,
        flip_h=False,
    )
    f_hash = img_hash.image_hash_file()
    assert ImagePHash.hash_to_str(f_hash) == "0"
    img_hash = ImagePHash(
        filename="./tests/datas/imageWhite.png",
        mode="pHash",
        flip_v=False,
        flip_h=False,
    )
    f_hash = img_hash.image_hash_file()
    assert ImagePHash.hash_to_str(f_hash) == "0"
    img_hash = ImagePHash(
        filename="./tests/datas/imageWhiteCircle1.png",
        mode="pHash",
        flip_v=False,
        flip_h=False,
    )
    f_hash = img_hash.image_hash_file()
    assert ImagePHash.hash_to_str(f_hash) == "4340418648956181752"


def test_averageHash(make_images):
    assert make_images == True
    img_hash = ImagePHash(
        filename="./tests/datas/imageBlackRectangle2.png",
        mode="averageHash",
        flip_v=False,
        flip_h=False,
    )
    f_hash = img_hash.image_hash_file()
    assert ImagePHash.hash_to_str(f_hash) == "26491358281728"
    assert (
        ImagePHash.hash_to_str(
            ImagePHash.str_to_hash(
                ImagePHash.hash_to_str(f_hash), "averageHash"
            )
        )
        == "26491358281728"
    )


def test_blockMeanHash(make_images):
    assert make_images == True
    img_hash = ImagePHash(
        filename="./tests/datas/imageBlackRectangle2.png",
        mode="blockMeanHash",
        flip_v=False,
        flip_h=False,
    )
    f_hash = img_hash.image_hash_file()
    assert (
        ImagePHash.hash_to_str(f_hash)
        == "4708568615394428010519620309870212934089941258315990302720"
    )
    assert (
        ImagePHash.hash_to_str(
            ImagePHash.str_to_hash(
                ImagePHash.hash_to_str(f_hash), "blockMeanHash"
            )
        )
        == "4708568615394428010519620309870212934089941258315990302720"
    )


def test_marrHildrethHash(make_images):
    assert make_images == True
    img_hash = ImagePHash(
        filename="./tests/datas/imageBlackRectangle2.png",
        mode="marrHildrethHash",
        flip_v=False,
        flip_h=False,
    )
    f_hash = img_hash.image_hash_file()
    assert (
        ImagePHash.hash_to_str(f_hash)
        == "25482809819799752521272967904782465118405599539750524436738731381942189100914021150153166941243469212048728742286905172028806218439071928825474226728864603222210835837455951"
        or ImagePHash.hash_to_str(f_hash)
        == "150270835317669957762723653744460230143878919778699801513548337344892614900664649703115090262454622976337584590297974955283386254333524992123949586010279903474896900699701552"
    )
    assert (
        ImagePHash.hash_to_str(
            ImagePHash.str_to_hash(
                ImagePHash.hash_to_str(f_hash), "marrHildrethHash"
            )
        )
        == "25482809819799752521272967904782465118405599539750524436738731381942189100914021150153166941243469212048728742286905172028806218439071928825474226728864603222210835837455951"
        or ImagePHash.hash_to_str(
            ImagePHash.str_to_hash(
                ImagePHash.hash_to_str(f_hash), "marrHildrethHash"
            )
        )
        == "150270835317669957762723653744460230143878919778699801513548337344892614900664649703115090262454622976337584590297974955283386254333524992123949586010279903474896900699701552"
    )


def test_radialVarianceHash(make_images):
    assert make_images == True
    img_hash = ImagePHash(
        filename="./tests/datas/imageBlackRectangle2.png",
        mode="radialVarianceHash",
        flip_v=False,
        flip_h=False,
    )
    f_hash = img_hash.image_hash_file()
    assert (
        ImagePHash.hash_to_str(f_hash)
        == "701098954365952339631195776418226034447310628990253611246130361972980936340383745096279972337493"
    )
    assert (
        ImagePHash.hash_to_str(
            ImagePHash.str_to_hash(
                ImagePHash.hash_to_str(f_hash), "radialVarianceHash"
            )
        )
        == "701098954365952339631195776418226034447310628990253611246130361972980936340383745096279972337493"
    )


def test_colorMomentHash(make_images):
    assert make_images == True
    img_hash = ImagePHash(
        filename="./tests/datas/imageBlackRectangle2.png",
        mode="colorMomentHash",
        flip_v=False,
        flip_h=False,
    )
    f_hash = img_hash.image_hash_file()
    assert (
        ImagePHash.hash_to_str(f_hash)
        == "20022558097416197357142269234771802705946225291146232497241770230660849506924986917791492459958055675893195011818104653717565431437649375187263170404774220901633534769349442160666321938577675689520828274390767765326457783021192785244043886215835318110716985098107829734864257266869443093758574436161166574847903164543307246799779062712216571103444626767987453813663436424148736705081494968276986531563948306884435734721233362387083457715563285871604675244273972519648273108409513662517244957751504246025626190955471276737309231802943046960796257414511296087096501511806649924251906898877691922923870376211237069019863992166855215469714073263635456671464083768518372830592846258199670511832674718852800403528211847323385445311615643167559257461588360279316824542120218851399918099751715239767405386223288773177"
        or "66574847906290472275097079330810033531430058235059661718187756560411474872672123142300706088030362495065935660709175031062566584888342571445611915336822260400194420050946504083648334869518964491887202131729618798219392531367010120327168441734087775458093671610225425888586681671319734590429343766088356090356461899440459787391240333226541499249128699886657759728493147612986366681057310316999967200524288328429610461229428541273417028415995728871303561707701620739073027577034718631205508391243550110388513337"
        or ImagePHash.hash_to_str(f_hash)
        == "20022558097416197357142269234771802705946225291146232497241770230660849506924986917791492459958055675893195011818104653717565431437649375187263170404774220901633534769349442160666321938577675689520828274390767765326457783021192785244043886215835318110716985098107829734864257266869443093758574436161166574847906290472275097079330810033531430058235059661718187756560411474872672123142300706088030362495065935660709175031062566584888342571445611915336822260400194420050946504083648334869518964491887202131729618798219392531367010120327168441734087775458093671610225425888586681671319734590429343766088356090356461899440459787391240333226541499249128699886657759728493147612986366681057310316999967200524288328429610461229428541273417028415995728871303561707701620739073027577034718631205508391243550110388513337"
        or "66574847906290472275097079330810033531430058235059661718187756560411474872672123142300706088030362495065935660709175031062566584888342571445611915336822260400194420050946504083648334869518964491887202131729618798219392531367010120327168441734087775458093671610225425888586681671319734590429343766088356090356461899440459787391240333226541499249128699886657759728493147612986366681057310316999967200524288328429610461229428541273417028415995728871303561707701620739073027577034718631205508391243550110388513337"
    )
    with pytest.raises(Exception):
        ImagePHash.hash_to_str(
            ImagePHash.str_to_hash(
                ImagePHash.hash_to_str(f_hash), "colorMomentHash"
            )
        )


def test_flip_v(make_images):
    assert make_images == True
    img_hash = ImagePHash(
        filename="./tests/datas/imageBlackRectangle2.png",
        mode="pHash",
        flip_v=True,
        flip_h=False,
    )
    f_hash = img_hash.image_hash_file()
    assert ImagePHash.hash_to_str(f_hash) == "11067990150089382910"
    assert len(img_hash.hash) == 2
    assert ImagePHash.hash_to_str(img_hash.hash[1]) == "11053635145317129211"


def test_flip_h(make_images):
    assert make_images == True
    img_hash = ImagePHash(
        filename="./tests/datas/imageBlackRectangle2.png",
        mode="pHash",
        flip_v=False,
        flip_h=True,
    )
    f_hash = img_hash.image_hash_file()
    assert ImagePHash.hash_to_str(f_hash) == "11067990150089382910"
    assert len(img_hash.hash) == 2
    assert ImagePHash.hash_to_str(img_hash.hash[1]) == "3689517699298472958"


def test_flip_v_h(make_images):
    assert make_images == True
    img_hash = ImagePHash(
        filename="./tests/datas/imageBlackRectangle2.png",
        mode="pHash",
        flip_v=True,
        flip_h=True,
    )
    f_hash = img_hash.image_hash_file()
    assert ImagePHash.hash_to_str(f_hash) == "11067990150089382910"
    assert len(img_hash.hash) == 4
    assert ImagePHash.hash_to_str(img_hash.hash[1]) == "11053635145317129211"
    assert ImagePHash.hash_to_str(img_hash.hash[2]) == "3732582713615234043"
    assert ImagePHash.hash_to_str(img_hash.hash[3]) == "3689517699298472958"


def test_distance(make_images):
    assert make_images == True
    img_hash = ImagePHash(
        filename="./tests/datas/imageBlackRectangle2.png",
        mode="pHash",
        flip_v=True,
        flip_h=True,
    )
    f_hash = img_hash.image_hash_file()
    assert ImagePHash.hash_to_str(f_hash) == "11067990150089382910"
    assert len(img_hash.hash) == 4
    assert (
        ImagePHash.hamming_distance_v3(img_hash.hash[0], img_hash.hash[1])
        == 26
    )
    assert (
        ImagePHash.hamming_distance_v2(img_hash.hash[0], img_hash.hash[2])
        == 27
    )
    assert (
        ImagePHash.hamming_distance(img_hash.hash[0], img_hash.hash[3]) == 25.0
    )
    assert (
        ImagePHash.hamming_distance_v2(img_hash.hash[1], img_hash.hash[0])
        == 26
    )
    assert (
        ImagePHash.hamming_distance_v3(img_hash.hash[1], img_hash.hash[2])
        == 25
    )
    assert (
        ImagePHash.hamming_distance(img_hash.hash[1], img_hash.hash[3]) == 27.0
    )


def test_min_distance(make_images):
    assert make_images == True
    img_hash = ImagePHash(
        filename="./tests/datas/imageBlackRectangle2.png",
        mode="pHash",
        flip_v=True,
        flip_h=True,
    )
    img_hash.image_hash_file()
    img_hash2 = ImagePHash(
        filename="./tests/datas/imageBlackRectangle1.png",
        mode="pHash",
        flip_v=True,
        flip_h=True,
    )
    img_hash2.image_hash_file()
    assert (
        ImagePHash.min_hash_distance(
            img_hash.hash, img_hash2.hash, verbose=True
        )
        == 18.0
    )
